from spaceone.core.pygrpc import BaseAPI
from spaceone.api.identity.v2 import workspace_pb2, workspace_pb2_grpc
from spaceone.identity.service.workspace_service import WorkspaceService


class Workspace(BaseAPI, workspace_pb2_grpc.WorkspaceServicer):
    pb2 = workspace_pb2
    pb2_grpc = workspace_pb2_grpc

    def create(self, request, context):
        params, metadata = self.parse_request(request, context)
        workspace_svc = WorkspaceService(metadata)
        response: dict = workspace_svc.create(params)
        return self.dict_to_message(response)

    def update(self, request, context):
        params, metadata = self.parse_request(request, context)
        workspace_svc = WorkspaceService(metadata)
        response: dict = workspace_svc.update(params)
        return self.dict_to_message(response)

    def delete(self, request, context):
        params, metadata = self.parse_request(request, context)
        workspace_svc = WorkspaceService(metadata)
        workspace_svc.delete(params)
        return self.empty()

    def enable(self, request, context):
        params, metadata = self.parse_request(request, context)
        workspace_svc = WorkspaceService(metadata)
        response: dict = workspace_svc.enable(params)
        return self.dict_to_message(response)

    def disable(self, request, context):
        params, metadata = self.parse_request(request, context)
        workspace_svc = WorkspaceService(metadata)
        response: dict = workspace_svc.disable(params)
        return self.dict_to_message(response)

    def get(self, request, context):
        params, metadata = self.parse_request(request, context)
        workspace_svc = WorkspaceService(metadata)
        response: dict = workspace_svc.get(params)
        return self.dict_to_message(response)

    def check(self, request, context):
        params, metadata = self.parse_request(request, context)
        workspace_svc = WorkspaceService(metadata)
        workspace_svc.check(params)
        return self.empty()

    def list(self, request, context):
        params, metadata = self.parse_request(request, context)
        workspace_svc = WorkspaceService(metadata)
        response: dict = workspace_svc.list(params)
        return self.dict_to_message(response)

    def stat(self, request, context):
        params, metadata = self.parse_request(request, context)
        workspace_svc = WorkspaceService(metadata)
        response: dict = workspace_svc.stat(params)
        return self.dict_to_message(response)



from typing import Union, Literal
from pydantic import BaseModel

__all__ = [
    "AdminUser",
    "DomainCreateRequest",
    "DomainUpdateRequest",
    "DomainDeleteRequest",
    "DomainEnableRequest",
    "DomainDisableRequest",
    "DomainGetRequest",
    "DomainGetAuthInfoRequest",
    "DomainGetPublicKeyRequest",
    "DomainSearchQueryRequest",
    "DomainStatQueryRequest",
    "State",
]

State = Literal["ENABLED", "DISABLED"]


class AdminUser(BaseModel):
    user_id: str
    password: str
    name: str
    email: Union[str, None] = None
    language: Union[str, None] = None
    timezone: Union[str, None] = None
    tags: Union[dict, None] = None


class DomainCreateRequest(BaseModel):
    name: str
    admin: AdminUser
    tags: Union[dict, None] = None


class DomainUpdateRequest(BaseModel):
    domain_id: str
    name: Union[str, None] = None
    tags: Union[dict, None] = None


class DomainDeleteRequest(BaseModel):
    domain_id: str


class DomainEnableRequest(BaseModel):
    domain_id: str


class DomainDisableRequest(BaseModel):
    domain_id: str


class DomainGetRequest(BaseModel):
    domain_id: str


class DomainGetAuthInfoRequest(BaseModel):
    name: str


class DomainGetPublicKeyRequest(BaseModel):
    domain_id: str


class DomainSearchQueryRequest(BaseModel):
    query: Union[dict, None] = None
    domain_id: Union[str, None] = None
    name: Union[str, None] = None
    state: Union[State, None] = None


class DomainStatQueryRequest(BaseModel):
    query: dict



import logging

from spaceone.identity.error.error_authentication import *
from spaceone.identity.error.error_user import ERROR_USER_STATE_DISABLED
from spaceone.identity.lib.cipher import PasswordCipher
from spaceone.identity.manager.user_manager import UserManager
from spaceone.identity.manager.token_manager.base import TokenManager

_LOGGER = logging.getLogger(__name__)


class LocalTokenManager(TokenManager):
    auth_type = "LOCAL"

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.user_mgr = UserManager()

    def authenticate(self, domain_id, **kwargs):
        self._check_credentials(kwargs)
        user_id = kwargs["user_id"]
        password = kwargs["password"]

        self.user = self.user_mgr.get_user(user_id, domain_id)

        self._check_user_state()

        # TODO: decrypt pw
        is_correct = PasswordCipher().checkpw(password, self.user.password)
        _LOGGER.debug(f"[authenticate] is_correct: {is_correct}")

        if is_correct:
            self.is_authenticated = True

            if self.user.state == "PENDING":
                self.user_mgr.update_user_by_vo({"state": "ENABLED"}, self.user)

        else:
            raise ERROR_AUTHENTICATION_FAILURE(user_id=self.user.user_id)

    def _check_user_state(self):
        if self.user.state == "DISABLED":
            raise ERROR_USER_STATE_DISABLED(user_id=self.user.user_id)

        if self.user.auth_type != "LOCAL":
            raise ERROR_NOT_FOUND(key="user_id", value=self.user.user_id)

    @staticmethod
    def _check_credentials(credentials):
        if credentials.get("user_id") is None:
            raise ERROR_INVALID_CREDENTIALS()

        if credentials.get("password") is None:
            raise ERROR_INVALID_CREDENTIALS()



from spaceone.core.pygrpc import BaseAPI
from spaceone.api.identity.v2 import system_pb2, system_pb2_grpc
from spaceone.identity.service.system_service import SystemService


class System(BaseAPI, system_pb2_grpc.SystemServicer):
    pb2 = system_pb2
    pb2_grpc = system_pb2_grpc

    def init(self, request, context):
        params, metadata = self.parse_request(request, context)
        system_svc = SystemService(metadata)
        response: dict = system_svc.init(params)
        return self.dict_to_message(response)



import logging
from typing import Tuple

from spaceone.core.manager import BaseManager
from spaceone.core import utils

from spaceone.identity.lib.key_generator import KeyGenerator
from spaceone.identity.manager.domain_secret_manager import DomainSecretManager

_LOGGER = logging.getLogger(__name__)


class APIKeyManager(BaseManager):
    @staticmethod
    def generate_api_key(
        app_id: str,
        domain_id: str,
        expired_at: str,
        role_type: str,
        workspace_id: str = None,
        permissions: list = None,
    ) -> Tuple[str, str]:
        domain_secret_mgr = DomainSecretManager()
        prv_jwk = domain_secret_mgr.get_domain_private_key(domain_id)
        refresh_prv_jwk = domain_secret_mgr.get_domain_refresh_private_key(domain_id)
        api_key_id = utils.generate_id("api-key")

        key_gen = KeyGenerator(
            prv_jwk,
            domain_id,
            "APP",
            app_id,
            api_key_id=api_key_id,
            refresh_prv_jwk=refresh_prv_jwk,
        )

        api_key = key_gen.generate_token(
            "API_KEY",
            expired_at=expired_at,
            role_type=role_type,
            workspace_id=workspace_id,
            permissions=permissions,
        )

        return api_key_id, api_key



import logging

from google.protobuf.json_format import MessageToDict
from spaceone.core import pygrpc
from spaceone.core.connector import BaseConnector
from spaceone.core.utils import parse_grpc_endpoint

from spaceone.identity.error.error_authentication import *

_LOGGER = logging.getLogger(__name__)


class AuthPluginConnector(BaseConnector):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.client = None

    def initialize(self, endpoint):
        static_endpoint = self.config.get("endpoint")

        if static_endpoint:
            endpoint = static_endpoint

        _LOGGER.info(f"[initialize] endpoint: {endpoint}")

        e = parse_grpc_endpoint(endpoint)
        self.client = pygrpc.client(
            endpoint=e["endpoint"], ssl_enabled=e["ssl_enabled"]
        )

    def call_login(self, credentials, options, secret_data, schema=None):
        params = {
            "options": options,
            "secret_data": secret_data,
            "schema": schema,
            "user_credentials": credentials,
        }

        try:
            response = self.client.Auth.login(
                params, metadata=self.transaction.get_connection_meta()
            )
        except ERROR_BASE as e:
            _LOGGER.error(f"[call_login] Auth.login failed. (reason={e.message})")
            raise ERROR_INVALID_CREDENTIALS()
        except Exception as e:
            _LOGGER.error(f"[call_login] Auth.login failed. (reason={str(e)})")
            raise ERROR_INVALID_CREDENTIALS()

        return MessageToDict(response, preserving_proto_field_name=True)

    def init(self, options: dict):
        params = {"options": options}

        try:
            response = self.client.Auth.init(
                params, metadata=self.transaction.get_connection_meta()
            )
            return MessageToDict(response, preserving_proto_field_name=True)
        except ERROR_BASE as e:
            raise ERROR_AUTHENTICATION_FAILURE_PLUGIN(message=e.message)
        except Exception as e:
            raise ERROR_AUTHENTICATION_FAILURE_PLUGIN(messsage=str(e))

    def verify(self, options, secret_data=None, schema=None):
        params = {"options": options, "secret_data": secret_data, "schema": schema}
        try:
            # TODO: meta (plugin has no meta)
            response = self.client.Auth.verify(
                params, metadata=self.transaction.get_connection_meta()
            )
            return MessageToDict(response, preserving_proto_field_name=True)
        except ERROR_BASE as e:
            raise ERROR_AUTHENTICATION_FAILURE_PLUGIN(message=e.message)
        except Exception as e:
            raise ERROR_AUTHENTICATION_FAILURE_PLUGIN(messsage=str(e))

    def call_find(self, keyword, user_id, options, secret_data={}, schema=None):
        params = {
            "options": options,
            "secret_data": secret_data,
            "schema": schema,
            "keyword": keyword,
            "user_id": user_id,
        }
        _LOGGER.info(f"[call_find] params: {params}")

        try:
            response = self.client.Auth.find(
                params, metadata=self.transaction.get_connection_meta()
            )

            users_info = MessageToDict(response, preserving_proto_field_name=True)
            _LOGGER.debug(f"[call_find] MessageToDict(user_info): {users_info}")
            return users_info

        except ERROR_BASE as e:
            raise ERROR_AUTHENTICATION_FAILURE_PLUGIN(message=e.message)
        except Exception as e:
            raise ERROR_AUTHENTICATION_FAILURE_PLUGIN(messsage=str(e))



from typing import Union, Literal
from pydantic import BaseModel

__all__ = [
    "TrustedAccountCreateRequest",
    "TrustedAccountUpdateRequest",
    "TrustedAccountUpdateSecretRequest",
    "TrustedAccountDeleteRequest",
    "TrustedAccountGetRequest",
    "TrustedAccountSearchQueryRequest",
    "TrustedAccountStatQueryRequest",
    "ResourceGroup",
]

ResourceGroup = Literal["DOMAIN", "WORKSPACE"]


class TrustedAccountCreateRequest(BaseModel):
    name: str
    data: dict
    provider: str
    secret_schema_id: str
    secret_data: dict
    tags: Union[dict, None] = None
    resource_group: ResourceGroup
    workspace_id: Union[str, None] = None
    domain_id: str


class TrustedAccountUpdateRequest(BaseModel):
    trusted_account_id: str
    name: Union[str, None] = None
    data: Union[dict, None] = None
    tags: Union[dict, None] = None
    workspace_id: str
    domain_id: str


class TrustedAccountUpdateSecretRequest(BaseModel):
    trusted_account_id: str
    secret_schema_id: str
    secret_data: dict
    workspace_id: Union[str, None] = None
    domain_id: str


class TrustedAccountDeleteRequest(BaseModel):
    trusted_account_id: str
    workspace_id: Union[str, None] = None
    domain_id: str


class TrustedAccountGetRequest(BaseModel):
    trusted_account_id: str
    workspace_id: Union[str, list, None] = None
    domain_id: str


class TrustedAccountSearchQueryRequest(BaseModel):
    query: Union[dict, None] = None
    trusted_account_id: Union[str, None] = None
    name: Union[str, None] = None
    provider: Union[str, None] = None
    secret_schema_id: Union[str, None] = None
    trusted_secret_id: Union[str, None] = None
    workspace_id: Union[str, list, None] = None
    domain_id: str


class TrustedAccountStatQueryRequest(BaseModel):
    query: dict
    workspace_id: Union[str, list, None] = None
    domain_id: str



import logging

from spaceone.core.manager import BaseManager
from spaceone.core.connector.space_connector import SpaceConnector

_LOGGER = logging.getLogger(__name__)


class SecretManager(BaseManager):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.secret_conn: SpaceConnector = self.locator.get_connector(
            "SpaceConnector", service="secret"
        )

    def create_trusted_secret(self, params: dict) -> dict:
        return self.secret_conn.dispatch("TrustedSecret.create", params)

    def update_trusted_secret_data(
        self, trusted_secret_id: str, schema_id: str, data: dict
    ) -> None:
        self.secret_conn.dispatch(
            "TrustedSecret.update_data",
            {
                "trusted_secret_id": trusted_secret_id,
                "schema_id": schema_id,
                "data": data,
            },
        )

    def delete_related_trusted_secrets(self, trusted_account_id: str):
        response = self.list_trusted_secrets({"trusted_account_id": trusted_account_id})

        for trusted_secret_info in response.get("results", []):
            trusted_secret_id = trusted_secret_info["trusted_secret_id"]
            self.secret_conn.dispatch(
                "TrustedSecret.delete",
                {"trusted_secret_id": trusted_secret_id},
            )

    def list_trusted_secrets(self, params: dict) -> dict:
        return self.secret_conn.dispatch("TrustedSecret.list", params)

    def create_secret(self, params: dict) -> dict:
        return self.secret_conn.dispatch("Secret.create", params)

    def update_secret(self, params: dict) -> dict:
        return self.secret_conn.dispatch("Secret.update", params)

    def delete_secret(self, secret_id: str) -> None:
        self.secret_conn.dispatch("Secret.delete", {"secret_id": secret_id})

    def update_secret_data(self, secret_id: str, schema_id: str, data: dict) -> None:
        self.secret_conn.dispatch(
            "Secret.update_data",
            {"secret_id": secret_id, "schema_id": schema_id, "data": data},
        )

    def delete_related_secrets(self, service_account_id: str):
        response = self.list_secrets({"service_account_id": service_account_id})

        for secret_info in response.get("results", []):
            secret_id = secret_info["secret_id"]
            self.secret_conn.dispatch(
                "Secret.delete",
                {"secret_id": secret_id},
            )

    def list_secrets(self, params: dict) -> dict:
        return self.secret_conn.dispatch("Secret.list", params)



import logging
from datetime import datetime
from spaceone.identity.connector.auth_plugin_connector import AuthPluginConnector
from spaceone.identity.error.error_authentication import *
from spaceone.identity.error.error_user import ERROR_USER_STATUS_CHECK_FAILURE
from spaceone.identity.manager.external_auth_manager import ExternalAuthManager
from spaceone.identity.manager.domain_manager import DomainManager
from spaceone.identity.manager.user_manager import UserManager
from spaceone.identity.manager.token_manager.base import JWTManager
from spaceone.identity.model.external_auth.database import ExternalAuth
from spaceone.identity.model.domain.database import Domain
from spaceone.identity.model.user.database import User

_LOGGER = logging.getLogger(__name__)


class ExternalTokenManager(JWTManager):
    domain: Domain = None
    external_auth: ExternalAuth = None
    auth_type = "EXTERNAL"

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.domain_mgr = DomainManager()
        self.external_auth_mgr = ExternalAuthManager
        self.user_mgr = UserManager()

    def authenticate(self, user_id: str, domain_id: str, credentials: dict):
        _LOGGER.debug(f"[authenticate] domain_id: {domain_id}")

        # Add User ID for External Authentication
        if user_id:
            credentials["user_id"] = user_id

        self.domain: Domain = self.domain_mgr.get_domain(domain_id)

        self._check_domain_state()

        self.external_auth = self.external_auth_mgr.get_external_auth(domain_id)
        endpoint, version = self.external_auth_mgr.get_auth_plugin_endpoint(
            self.domain.domain_id, self.external_auth.plugin_info
        )
        auth_user_info = self._authenticate_with_plugin(endpoint, credentials)

        _LOGGER.info(
            f'[authenticate] Authentication success. (user_id={auth_user_info.get("user_id")})'
        )

        auto_user_sync = self.external_auth.plugin_info.options.get(
            "auto_user_sync", False
        )

        self._verify_user_from_plugin_user_info(
            auth_user_info, domain_id, auto_user_sync
        )
        self._check_user_state()

        self.is_authenticated = True

    def issue_token(self, **kwargs):
        if self.is_authenticated is False:
            raise ERROR_NOT_AUTHENTICATED()

        if self.user.state == "PENDING":
            self.user: User = self.user.update({"state": "ENABLED"})

        # Issue token
        access_token = self.issue_access_token(
            "USER", self.user.user_id, self.user.domain_id, **kwargs
        )
        refresh_token = self.issue_refresh_token(
            "USER", self.user.user_id, self.user.domain_id, **kwargs
        )

        # Update user's last_accessed_at field
        self.user.update({"last_accessed_at": datetime.utcnow()})

        return {"access_token": access_token, "refresh_token": refresh_token}

    def refresh_token(self, user_id, domain_id, **kwargs):
        self.user: User = self.user_mgr.get_user(user_id, domain_id)
        self._check_user_state()

        return self.issue_token(**kwargs)

    def _verify_user_from_plugin_user_info(
        self, auth_user_info: dict, domain_id: str, auto_user_sync: bool = False
    ) -> None:
        if "user_id" not in auth_user_info:
            _LOGGER.error(
                f"[_verify_user_from_plugin_user_info] does not return user_id from plugin user info."
            )
            raise ERROR_AUTHENTICATION_FAILURE_PLUGIN(
                message="plugin response is invalid."
            )

        user_id = auth_user_info["user_id"]
        state = auth_user_info.get("state", "ENABLED")

        user_vos = self.user_mgr.filter_users(user_id=user_id, domain_id=domain_id)

        if user_vos.count() > 0:
            self.user: User = user_vos[0]
        else:
            if auto_user_sync:
                name = auth_user_info.get("name")
                email = auth_user_info.get("email")
                self.user: User = self._create_external_user(
                    user_id, state, domain_id, name, email
                )
            else:
                raise ERROR_NOT_FOUND(key="user_id", value=user_id)

    def _authenticate_with_plugin(self, endpoint: str, credentials: dict) -> dict:
        options = self.external_auth.plugin_info.options

        auth_plugin_conn = AuthPluginConnector("AuthPluginConnector")
        auth_plugin_conn.initialize(endpoint)

        return auth_plugin_conn.call_login(credentials, options, {})

    def _check_domain_state(self):
        external_auth_info = self.external_auth_mgr.get_auth_info(domain_vo=self.domain)

        if external_auth_info.get("external_auth_state") != "ENABLED":
            _LOGGER.error(
                "[_get_token_manager] This domain does not allow external authentication."
            )
            raise ERROR_AUTHENTICATION_FAILURE(user_id=self.user.user_id)

    def _check_user_state(self) -> None:
        if self.user.state not in ["ENABLED", "PENDING"]:
            raise ERROR_USER_STATUS_CHECK_FAILURE(user_id=self.user.user_id)

        if self.user.auth_type != "EXTERNAL":
            raise ERROR_NOT_FOUND(key="user_id", value=self.user.user_id)

    def _create_external_user(
        self,
        user_id: str,
        state: str,
        domain_id: str,
        name: str = None,
        email: str = None,
    ) -> User:
        _LOGGER.error(f"[_create_external_user] create user on first login: {user_id}")
        return self.user_mgr.create_user(
            {
                "user_id": user_id,
                "state": state,
                "name": name,
                "email": email,
                "auth_type": "EXTERNAL",
                "domain_id": domain_id,
            }
        )



from datetime import datetime
from typing import Union, Literal
from pydantic import BaseModel

__all__ = [
    "ServiceAccountCreateRequest",
    "ServiceAccountUpdateRequest",
    "ServiceAccountUpdateSecretRequest",
    "ServiceAccountDeleteSecretRequest",
    "ServiceAccountDeleteRequest",
    "ServiceAccountGetRequest",
    "ServiceAccountSearchQueryRequest",
    "ServiceAccountStatQueryRequest",
]


class ServiceAccountCreateRequest(BaseModel):
    name: str
    data: dict
    provider: str
    secret_schema_id: Union[str, None] = None
    secret_data: Union[dict, None] = None
    tags: Union[dict, None] = None
    trusted_account_id: Union[str, None] = None
    project_id: str
    workspace_id: str
    domain_id: str


class ServiceAccountUpdateRequest(BaseModel):
    service_account_id: str
    name: Union[str, None] = None
    data: Union[dict, None] = None
    tags: Union[dict, None] = None
    project_id: Union[str, None] = None
    workspace_id: str
    domain_id: str
    user_projects: Union[list, None] = None


class ServiceAccountUpdateSecretRequest(BaseModel):
    service_account_id: str
    secret_schema_id: str
    secret_data: dict
    trusted_account_id: Union[str, None] = None
    workspace_id: str
    domain_id: str
    user_projects: Union[list, None] = None


class ServiceAccountDeleteSecretRequest(BaseModel):
    service_account_id: str
    workspace_id: str
    domain_id: str
    user_projects: Union[list, None] = None


class ServiceAccountDeleteRequest(BaseModel):
    service_account_id: str
    workspace_id: str
    domain_id: str
    user_projects: Union[list, None] = None


class ServiceAccountGetRequest(BaseModel):
    service_account_id: str
    workspace_id: Union[str, None] = None
    domain_id: str
    user_projects: Union[list, None] = None


class ServiceAccountSearchQueryRequest(BaseModel):
    query: Union[dict, None] = None
    service_account_id: Union[str, None] = None
    name: Union[str, None] = None
    provider: Union[str, None] = None
    secret_schema_id: Union[str, None] = None
    secret_id: Union[str, None] = None
    trusted_account_id: Union[str, None] = None
    project_id: Union[str, None] = None
    workspace_id: Union[str, None] = None
    domain_id: str
    user_projects: Union[list, None] = None


class ServiceAccountStatQueryRequest(BaseModel):
    query: dict
    workspace_id: Union[str, None] = None
    domain_id: str
    user_projects: Union[list, None] = None



import logging
from datetime import datetime, timedelta
from typing import Union

from spaceone.core.service import *
from spaceone.core.service.utils import *

from spaceone.identity.error.error_api_key import *
from spaceone.identity.manager.app_manager import AppManager
from spaceone.identity.manager.workspace_manager import WorkspaceManager
from spaceone.identity.manager.role_manager import RoleManager
from spaceone.identity.manager.api_key_manager import APIKeyManager
from spaceone.identity.manager.domain_manager import DomainManager
from spaceone.identity.model.app.request import *
from spaceone.identity.model.app.response import *
from spaceone.identity.error.error_role import ERROR_NOT_ALLOWED_ROLE_TYPE

_LOGGER = logging.getLogger(__name__)


@authentication_handler
@authorization_handler
@mutation_handler
@event_handler
class AppService(BaseService):
    resource = "App"

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.app_mgr = AppManager()

    @transaction(
        permission="identity:App.write",
        role_types=["DOMAIN_ADMIN", "WORKSPACE_OWNER"],
    )
    @convert_model
    def create(self, params: AppCreateRequest) -> Union[AppResponse, dict]:
        """Create API Key
        Args:
            params (AppCreateRequest): {
                'name': 'str',              # required
                'role_id': 'str',           # required
                'tags': 'dict',
                'expired_at': 'str',
                'resource_group': 'str',    # required
                'workspace_id': 'str',      # injected from auth
                'domain_id': 'str',         # injected from auth (required)
            }
        Return:
            AppResponse:
        """

        # Check role
        role_mgr = RoleManager()
        role_vo = role_mgr.get_role(params.role_id, params.domain_id)

        # Check workspace
        if params.resource_group == "WORKSPACE":
            if params.workspace_id is None:
                raise ERROR_REQUIRED_PARAMETER(key="workspace_id")

            workspace_mgr = WorkspaceManager()
            workspace_mgr.get_workspace(params.workspace_id, params.domain_id)
        else:
            params.workspace_id = "*"

        # Check role type by resource_group
        if params.resource_group == "DOMAIN":
            if role_vo.role_type != "DOMAIN_ADMIN":
                raise ERROR_NOT_ALLOWED_ROLE_TYPE(
                    request_role_id=role_vo.role_id,
                    request_role_type=role_vo.role_type,
                    supported_role_type="DOMAIN_ADMIN",
                )
        else:
            if role_vo.role_type != "WORKSPACE_OWNER":
                raise ERROR_NOT_ALLOWED_ROLE_TYPE(
                    request_role_id=role_vo.role_id,
                    request_role_type=role_vo.role_type,
                    supported_role_type="WORKSPACE_OWNER",
                )

        params.expired_at = self._get_expired_at(params.expired_at)
        self._check_expired_at(params.expired_at)

        params_data = params.dict()
        params_data["role_type"] = role_vo.role_type

        app_vo = self.app_mgr.create_app(params_data)

        api_key_mgr = APIKeyManager()
        api_key_id, api_key = api_key_mgr.generate_api_key(
            app_vo.app_id,
            app_vo.domain_id,
            params.expired_at,
            app_vo.role_type,
            app_vo.workspace_id,
        )

        app_vo = self.app_mgr.update_app_by_vo({"api_key_id": api_key_id}, app_vo)

        return AppResponse(**app_vo.to_dict(), api_key=api_key)

    @transaction(
        permission="identity:App.write",
        role_types=["DOMAIN_ADMIN", "WORKSPACE_OWNER"],
    )
    @convert_model
    def update(self, params: AppUpdateRequest) -> Union[AppResponse, dict]:
        """Update App
        Args:
            params (dict): {
                'app_id': 'str',        # required
                'name': 'str',
                'tags': 'dict',
                'workspace_id': 'str',  # injected from auth
                'domain_id': 'str'      # injected from auth (required)
            }
        Return:
            AppResponse:
        """

        app_vo = self.app_mgr.get_app(
            params.app_id,
            params.domain_id,
            params.workspace_id,
        )
        app_vo = self.app_mgr.update_app_by_vo(params.dict(exclude_unset=True), app_vo)
        return AppResponse(**app_vo.to_dict())

    @transaction(
        permission="identity:App.write",
        role_types=["DOMAIN_ADMIN", "WORKSPACE_OWNER"],
    )
    @convert_model
    def generate_api_key(
        self, params: AppGenerateAPIKeyRequest
    ) -> Union[AppResponse, dict]:
        """Generate API Key
        Args:
            params (dict): {
                'app_id': 'str',        # required
                'expired_at': 'str',
                'workspace_id': 'str',  # injected from auth
                'domain_id': 'str'      # injected from auth (required)
            }
        Return:
            AppResponse:
        """

        params.expired_at = self._get_expired_at(params.expired_at)
        self._check_expired_at(params.expired_at)

        app_vo = self.app_mgr.get_app(
            params.app_id,
            params.domain_id,
            params.workspace_id,
        )

        # Create new api_key
        api_key_mgr = APIKeyManager()
        api_key_id, api_key = api_key_mgr.generate_api_key(
            app_vo.app_id,
            app_vo.domain_id,
            params.expired_at,
            app_vo.role_type,
            app_vo.workspace_id,
        )

        # Update app info
        app_vo = self.app_mgr.update_app_by_vo({"api_key_id": api_key_id}, app_vo)

        return AppResponse(**app_vo.to_dict(), api_key=api_key)

    @transaction(
        permission="identity:App.write",
        role_types=["DOMAIN_ADMIN", "WORKSPACE_OWNER"],
    )
    @convert_model
    def enable(self, params: AppEnableRequest) -> Union[AppResponse, dict]:
        """Enable App Key
        Args:
            params (dict): {
                'app_id': 'str',        # required
                'workspace_id': 'str',  # injected from auth
                'domain_id': 'str'      # injected from auth (required)
            }
        """
        app_vo = self.app_mgr.get_app(
            params.app_id,
            params.domain_id,
            params.workspace_id,
        )
        app_vo = self.app_mgr.enable_app(app_vo)
        return AppResponse(**app_vo.to_dict())

    @transaction(
        permission="identity:App.write",
        role_types=["DOMAIN_ADMIN", "WORKSPACE_OWNER"],
    )
    @convert_model
    def disable(self, params: AppDisableRequest) -> Union[AppResponse, dict]:
        """Disable App Key
        Args:
            params (dict): {
                'app_id': 'str',        # required
                'workspace_id': 'str',  # injected from auth
                'domain_id': 'str'      # injected from auth (required)
            }
        """
        app_vo = self.app_mgr.get_app(
            params.app_id,
            params.domain_id,
            params.workspace_id,
        )
        app_vo = self.app_mgr.disable_app(app_vo)
        return AppResponse(**app_vo.to_dict())

    @transaction(
        permission="identity:App.write",
        role_types=["DOMAIN_ADMIN", "WORKSPACE_OWNER"],
    )
    @convert_model
    def delete(self, params: AppDeleteRequest) -> None:
        """Delete app
        Args:
            params (dict): {
                'api_key_id': 'str',    # required
                'workspace_id': 'str',  # injected from auth
                'domain_id': 'str'      # injected from auth (required)
            }
        Returns:
            None
        """
        app_vo = self.app_mgr.get_app(
            params.app_id,
            params.domain_id,
            params.workspace_id,
        )
        self.app_mgr.delete_app_by_vo(app_vo)

    @transaction(
        permission="identity:App.read",
        role_types=["DOMAIN_ADMIN", "WORKSPACE_OWNER"],
    )
    @convert_model
    def get(self, params: AppGetRequest) -> Union[AppResponse, dict]:
        """Get API Key
        Args:
            params (dict): {
                'app_id': 'str',            # required
                'workspace_id': 'list',     # injected from auth
                'domain_id': 'str'          # injected from auth (required)
            }
        Returns:
            AppResponse:
        """
        app_vo = self.app_mgr.get_app(
            params.app_id,
            params.domain_id,
            params.workspace_id,
        )
        return AppResponse(**app_vo.to_dict())

    @transaction(exclude=["authentication", "authorization", "mutation"])
    @convert_model
    def check(self, params: AppCheckRequest) -> Union[CheckAppResponse, dict]:
        """Get API Key
        Args:
            params (dict): {
                'api_key_id': 'str',        # required
                'domain_id': 'str'          # required
            }
        Returns:
            None:
        """

        app_vos = self.app_mgr.filter_apps(
            api_key_id=params.api_key_id,
            domain_id=params.domain_id,
        )

        if app_vos.count() == 0:
            raise ERROR_PERMISSION_DENIED()

        app_vo = app_vos[0]
        if app_vo.state != "ENABLED":
            raise ERROR_PERMISSION_DENIED()

        domain_mgr = DomainManager()
        domain_vo = domain_mgr.get_domain(app_vo.domain_id)
        if domain_vo.state != "ENABLED":
            raise ERROR_PERMISSION_DENIED()

        if app_vo.role_type == "WORKSPACE_OWNER":
            workspace_mgr = WorkspaceManager()
            workspace_vo = workspace_mgr.get_workspace(
                app_vo.workspace_id, app_vo.domain_id
            )
            if workspace_vo.state != "ENABLED":
                raise ERROR_PERMISSION_DENIED()

        role_mgr = RoleManager()
        role_vo = role_mgr.get_role(app_vo.role_id, app_vo.domain_id)
        return CheckAppResponse(permissions=role_vo.permissions)

    @transaction(
        permission="identity:App.read",
        role_types=["DOMAIN_ADMIN", "WORKSPACE_OWNER"],
    )
    @append_query_filter(
        [
            "app_id",
            "name",
            "state",
            "role_type",
            "role_id",
            "api_key_id",
            "workspace_id",
            "domain_id",
        ]
    )
    @append_keyword_filter(["app_id", "name"])
    @convert_model
    def list(self, params: AppSearchQueryRequest) -> Union[AppsResponse, dict]:
        """List Apps
        Args:
            params (dict): {
                'query': 'dict (spaceone.api.core.v1.Query)',
                'app_id': 'str',
                'name': 'str',
                'state': 'str',
                'role_type': 'str',
                'role_id': 'str',
                'api_key_id': 'str',
                'workspace_id': 'list'      # injected from auth
                'domain_id': 'str'          # injected from auth (required)
            }
        Returns:
            AppsResponse:
        """
        query = params.query or {}
        app_vos, total_count = self.app_mgr.list_apps(query)
        apps_info = [app_vo.to_dict() for app_vo in app_vos]
        return AppsResponse(results=apps_info, total_count=total_count)

    @transaction(
        permission="identity:App.read",
        role_types=["DOMAIN_ADMIN", "WORKSPACE_OWNER"],
    )
    @append_query_filter(["workspace_id", "domain_id"])
    @append_keyword_filter(["app_id", "name"])
    @convert_model
    def stat(self, params: AppStatQueryRequest) -> dict:
        """Stat API Keys
        Args:
            params (dict): {
                'query': 'dict',        # required
                'workspace_id': 'list', # injected from auth
                'domain_id': 'str'      # injected from auth (required)
            }
            Returns:
                dict:
        """
        query = params.query or {}
        return self.app_mgr.stat_apps(query)

    @staticmethod
    def _get_expired_at(expired_at: str) -> str:
        if expired_at:
            return expired_at
        else:
            return (datetime.utcnow() + timedelta(days=365)).strftime(
                "%Y-%m-%d %H:%M:%S"
            )

    @staticmethod
    def _check_expired_at(expired_at: str) -> None:
        one_year_later = datetime.utcnow() + timedelta(days=365)

        if one_year_later.strftime("%Y-%m-%d %H:%M:%S") < expired_at:
            raise ERROR_API_KEY_EXPIRED_LIMIT(expired_at=expired_at)



import logging
from typing import Union

from spaceone.core.service import *
from spaceone.core.service.utils import *
from spaceone.core.error import *

from spaceone.identity.model.service_account.request import *
from spaceone.identity.model.service_account.response import *
from spaceone.identity.manager.schema_manager import SchemaManager
from spaceone.identity.manager.service_account_manager import ServiceAccountManager
from spaceone.identity.manager.trusted_account_manager import TrustedAccountManager
from spaceone.identity.manager.project_manager import ProjectManager
from spaceone.identity.manager.secret_manager import SecretManager

_LOGGER = logging.getLogger(__name__)


@authentication_handler
@authorization_handler
@mutation_handler
@event_handler
class ServiceAccountService(BaseService):
    resource = "ServiceAccount"

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.service_account_mgr = ServiceAccountManager()

    @transaction(
        permission="identity:ServiceAccount.write",
        role_types=["WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @convert_model
    def create(
        self, params: ServiceAccountCreateRequest
    ) -> Union[ServiceAccountResponse, dict]:
        """create service account

         Args:
            params (ServiceAccountCreateRequest): {
                'name': 'str',                          # required
                'data': 'dict',                         # required
                'provider': 'str',                      # required
                'secret_schema_id': 'str',
                'secret_data': 'dict',
                'trusted_account_id': 'str',
                'tags': 'dict',
                'project_id': 'str',                    # required
                'workspace_id': 'str',                  # injected from auth (required)
                'domain_id': 'str'                      # injected from auth (required)
            }

        Returns:
            ServiceAccountResponse:
        """

        # Check data by schema
        schema_mgr = SchemaManager()
        schema_mgr.validate_data_by_schema(
            params.provider, params.domain_id, "SERVICE_ACCOUNT", params.data
        )

        # Check project
        project_mgr = ProjectManager()
        project_mgr.get_project(
            params.project_id, params.domain_id, params.workspace_id
        )

        # Check trusted service account
        if params.trusted_account_id and params.secret_data:
            trusted_account_mgr = TrustedAccountManager()

            # Append wildcard to workspace_id for DOMAIN trusted account
            workspace_id = [params.workspace_id, "*"]

            trusted_account_vo = trusted_account_mgr.get_trusted_account(
                params.trusted_account_id,
                params.domain_id,
                workspace_id,
            )
            secret_type = "TRUSTING_SECRET"
        else:
            trusted_account_vo = None
            params.trusted_account_id = None
            secret_type = "SECRET"

        service_account_vo = self.service_account_mgr.create_service_account(
            params.dict()
        )

        if params.secret_data:
            if params.secret_schema_id is None:
                raise ERROR_REQUIRED_PARAMETER(key="secret_schema_id")

            # Check secret_data by schema
            schema_mgr.validate_secret_data_by_schema_id(
                params.secret_schema_id,
                params.domain_id,
                params.secret_data,
                secret_type,
            )

            # Create a secret
            secret_mgr = SecretManager()
            create_secret_params = {
                "name": f"{service_account_vo.service_account_id}-secret",
                "data": params.secret_data,
                "schema_id": params.secret_schema_id,
                "service_account_id": service_account_vo.service_account_id,
                "resource_group": "PROJECT",
            }

            if trusted_account_vo:
                create_secret_params[
                    "trusted_secret_id"
                ] = trusted_account_vo.trusted_secret_id

            secret_info = secret_mgr.create_secret(create_secret_params)

            # Update secret_id in service_account_vo
            service_account_vo = self.service_account_mgr.update_service_account_by_vo(
                {"secret_id": secret_info["secret_id"]}, service_account_vo
            )

        return ServiceAccountResponse(**service_account_vo.to_dict())

    @transaction(
        permission="identity:ServiceAccount.write",
        role_types=["WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @convert_model
    def update(
        self, params: ServiceAccountUpdateRequest
    ) -> Union[ServiceAccountResponse, dict]:
        """update service account

         Args:
            params (ServiceAccountUpdateRequest): {
                'service_account_id': 'str',        # required
                'name': 'str',
                'data': 'dict',
                'tags': 'dict',
                'project_id': 'str',
                'workspace_id': 'str',              # injected from auth (required)
                'domain_id': 'str',                 # injected from auth (required)
                'user_projects': 'list'             # injected from auth
            }

        Returns:
            ServiceAccountResponse:
        """

        service_account_vo = self.service_account_mgr.get_service_account(
            params.service_account_id,
            params.domain_id,
            params.workspace_id,
            params.user_projects,
        )

        if params.data:
            # Check data by schema
            schema_mgr = SchemaManager()
            schema_mgr.validate_data_by_schema(
                service_account_vo.provider,
                params.domain_id,
                "SERVICE_ACCOUNT",
                params.data,
            )

        service_account_vo = self.service_account_mgr.update_service_account_by_vo(
            params.dict(exclude_unset=True), service_account_vo
        )

        return ServiceAccountResponse(**service_account_vo.to_dict())

    @transaction(
        permission="identity:ServiceAccount.write",
        role_types=["WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @convert_model
    def update_secret_data(
        self, params: ServiceAccountUpdateSecretRequest
    ) -> Union[ServiceAccountResponse, dict]:
        """update service account secret data

         Args:
            params (ServiceAccountUpdateSecretRequest): {
                'service_account_id': 'str',        # required
                'secret_schema_id': 'str',          # required
                'secret_data': 'dict',              # required
                'trusted_account_id': 'str',
                'workspace_id': 'str',              # injected from auth (required)
                'domain_id': 'str',                 # injected from auth (required)
                'user_projects': 'list'             # injected from auth
            }

        Returns:
            ServiceAccountResponse:
        """

        service_account_vo = self.service_account_mgr.get_service_account(
            params.service_account_id,
            params.domain_id,
            params.workspace_id,
            params.user_projects,
        )

        if params.trusted_account_id:
            trusted_account_mgr = TrustedAccountManager()

            # Append wildcard to workspace_id for DOMAIN trusted account
            workspace_id = [params.workspace_id, "*"]

            trusted_account_vo = trusted_account_mgr.get_trusted_account(
                params.trusted_account_id, params.domain_id, workspace_id
            )
            schema_type = "TRUSTING_SECRET"
        else:
            trusted_account_vo = None
            params.trusted_account_id = None
            schema_type = "SECRET"

        # Check secret_data by schema
        schema_mgr = SchemaManager()
        schema_mgr.validate_secret_data_by_schema_id(
            params.secret_schema_id,
            params.domain_id,
            params.secret_data,
            schema_type,
        )

        secret_mgr = SecretManager()

        # Delete old secret
        if service_account_vo.secret_id:
            secret_mgr.delete_secret(service_account_vo.secret_id)

        # Create New Secret
        create_secret_params = {
            "name": f"{service_account_vo.service_account_id}-secret",
            "data": params.secret_data,
            "schema_id": params.secret_schema_id,
            "service_account_id": service_account_vo.service_account_id,
            "resource_group": "PROJECT",
        }

        if trusted_account_vo:
            create_secret_params[
                "trusted_secret_id"
            ] = trusted_account_vo.trusted_secret_id

        secret_info = secret_mgr.create_secret(create_secret_params)

        params_data = params.dict(exclude_unset=True)
        params_data["secret_id"] = secret_info["secret_id"]
        params_data["secret_schema_id"] = params.secret_schema_id

        service_account_vo = self.service_account_mgr.update_service_account_by_vo(
            params_data, service_account_vo
        )

        return ServiceAccountResponse(**service_account_vo.to_dict())

    @transaction(
        permission="identity:ServiceAccount.write",
        role_types=["WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @convert_model
    def delete_secret_data(
        self, params: ServiceAccountDeleteSecretRequest
    ) -> ServiceAccountResponse:
        """delete service account secret data

         Args:
            params (ServiceAccountDeleteSecretRequest): {
                'service_account_id': 'str',            # required
                'workspace_id': 'str',                  # injected from auth (required)
                'domain_id': 'str',                     # injected from auth (required)
                'user_projects': 'list'                 # injected from auth
            }

        Returns:
            ServiceAccountResponse:
        """

        service_account_vo = self.service_account_mgr.get_service_account(
            params.service_account_id,
            params.domain_id,
            params.workspace_id,
            params.user_projects,
        )

        secret_mgr = SecretManager()
        secret_mgr.delete_related_secrets(service_account_vo.service_account_id)

        service_account_vo = self.service_account_mgr.update_service_account_by_vo(
            {"secret_id": None, "secret_schema_id": None, "trusted_account_id": None},
            service_account_vo,
        )

        return ServiceAccountResponse(**service_account_vo.to_dict())

    @transaction(
        permission="identity:ServiceAccount.write",
        role_types=["WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @convert_model
    def delete(self, params: ServiceAccountDeleteRequest) -> None:
        """delete service account

         Args:
            params (ServiceAccountDeleteRequest): {
                'service_account_id': 'str',            # required
                'workspace_id': 'str',                  # injected from auth (required)
                'domain_id': 'str',                     # injected from auth (required)
                'user_projects': 'list'                 # injected from auth
            }

        Returns:
            None
        """

        service_account_vo = self.service_account_mgr.get_service_account(
            params.service_account_id,
            params.domain_id,
            params.workspace_id,
            params.user_projects,
        )

        secret_mgr = SecretManager()
        secret_mgr.delete_related_secrets(service_account_vo.service_account_id)

        self.service_account_mgr.delete_service_account_by_vo(service_account_vo)

    @transaction(
        permission="identity:ServiceAccount.read",
        role_types=["DOMAIN_ADMIN", "WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @convert_model
    def get(
        self, params: ServiceAccountGetRequest
    ) -> Union[ServiceAccountResponse, dict]:
        """get service account

         Args:
            params (ServiceAccountDeleteRequest): {
                'service_account_id': 'str',            # required
                'workspace_id': 'str',                  # injected from auth
                'domain_id': 'str',                     # injected from auth (required)
                'user_projects': 'list'                 # injected from auth
            }

        Returns:
             ServiceAccountResponse:
        """

        service_account_vo = self.service_account_mgr.get_service_account(
            params.service_account_id,
            params.domain_id,
            params.workspace_id,
            params.user_projects,
        )

        return ServiceAccountResponse(**service_account_vo.to_dict())

    @transaction(
        permission="identity:ServiceAccount.read",
        role_types=["DOMAIN_ADMIN", "WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @append_query_filter(
        [
            "service_account_id",
            "name",
            "provider",
            "secret_schema_id",
            "secret_id",
            "project_id",
            "workspace_id",
            "domain_id",
            "user_projects",
        ]
    )
    @append_keyword_filter(["service_account_id", "name"])
    @set_query_page_limit(1000)
    @convert_model
    def list(
        self, params: ServiceAccountSearchQueryRequest
    ) -> Union[ServiceAccountsResponse, dict]:
        """list service accounts

        Args:
            params (ServiceAccountSearchQueryRequest): {
                'query': 'dict (spaceone.api.core.v1.Query)',
                'service_account_id': 'str',
                'name': 'str',
                'provider': 'str',
                'secret_schema_id': 'str',
                'secret_id': 'str',
                'project_id': 'str',
                'workspace_id': 'str',                  # injected from auth
                'domain_id': 'str',                     # injected from auth (required)
                'user_projects': 'list'                 # injected from auth
            }

        Returns:
            ServiceAccountsResponse:
        """

        query = params.query or {}
        (
            service_account_vos,
            total_count,
        ) = self.service_account_mgr.list_service_accounts(query)

        service_accounts_info = [
            service_account_vo.to_dict() for service_account_vo in service_account_vos
        ]
        return ServiceAccountsResponse(
            results=service_accounts_info, total_count=total_count
        )

    @transaction(
        permission="identity:ServiceAccount.read",
        role_types=["DOMAIN_ADMIN", "WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @append_query_filter(["workspace_id", "domain_id", "user_projects"])
    @append_keyword_filter(["service_account_id", "name"])
    @set_query_page_limit(1000)
    @convert_model
    def stat(self, params: ServiceAccountStatQueryRequest) -> dict:
        """stat service accounts

        Args:
            params (ServiceAccountStatQueryRequest): {
                'query': 'dict (spaceone.api.core.v1.StatisticsQuery)', # required
                'workspace_id': 'str',      # injected from auth
                'domain_id': 'str',         # injected from auth (required)
                'user_projects': 'list'     # injected from auth
            }

        Returns:
            dict: {
                'results': 'list',
                'total_count': 'int'
            }

        """

        query = params.query or {}
        return self.service_account_mgr.stat_service_accounts(query)



from mongoengine import *
from spaceone.core.model.mongo_model import MongoModel


class RoleBinding(MongoModel):
    role_binding_id = StringField(max_length=40, generate_id="rb", unique=True)
    role_type = StringField(
        max_length=20,
        choices=("DOMAIN_ADMIN", "WORKSPACE_OWNER", "WORKSPACE_MEMBER"),
    )
    user_id = StringField(max_length=255)
    role_id = StringField(max_length=40)
    resource_group = StringField(max_length=40, choices=("DOMAIN", "WORKSPACE"))
    workspace_id = StringField(max_length=40)
    domain_id = StringField(max_length=40)
    created_at = DateTimeField(auto_now_add=True)

    meta = {
        "updatable_fields": [
            "role_id",
            "role_type",
        ],
        "minimal_fields": [
            "role_binding_id",
            "role_type",
            "user_id",
            "role_id",
            "workspace_id",
        ],
        "indexes": [
            "role_type",
            "user_id",
            "role_id",
            "resource_group",
            "workspace_id",
            "domain_id",
        ],
    }



from datetime import datetime
from typing import Union, Literal
from pydantic import BaseModel, Field

__all__ = ["ExternalAuthResponse"]

State = Literal["ENABLED", "DISABLED"]


class Plugin(BaseModel):
    plugin_id: str
    version: Union[str, None] = None
    upgrade_mode: Union[str, None] = None
    options: Union[dict, None] = None
    schema_id: Union[str, None] = Field(None, alias="schema")
    metadata: dict
    secret_id: Union[dict, None] = None


class ExternalAuthResponse(BaseModel):
    domain_id: Union[str, None] = None
    state: Union[State, None] = None
    plugin_info: Union[Plugin, None] = None
    updated_at: Union[datetime, None] = None



from mongoengine import *
from spaceone.core.model.mongo_model import MongoModel


class Provider(MongoModel):
    provider = StringField(max_length=40, unique_with='domain_id')
    name = StringField(max_length=40)
    version = StringField(max_length=40, default=None, null=True)
    alias = StringField(max_length=40, default=None, null=True)
    color = StringField(max_length=7, default=None, null=True)
    icon = StringField(default=None, null=True)
    order = IntField(min_value=1, default=10)
    options = DictField(default=None)
    tags = DictField(default=None)
    is_managed = BooleanField(default=False)
    domain_id = StringField(max_length=40)
    created_at = DateTimeField(auto_now_add=True)
    updated_at = DateTimeField(auto_now=True)

    meta = {
        'updatable_fields': [
            'name',
            'version',
            'alias',
            'color',
            'icon',
            'order',
            'options',
            'tags',
            'updated_at'
        ],
        'minimal_fields': [
            'provider',
            'name',
            'order',
            'is_managed'
        ],
        'ordering': [
            'order',
            'name'
        ],
        'indexes': [
            'provider',
            'is_managed',
            'domain_id'
        ]
    }



from spaceone.core.error import *


class ERROR_SCHEMA_IS_NOT_DEFINED(ERROR_INVALID_ARGUMENT):
    _message = "Schema is not defined. (provider={provider}, schema_type={schema_type})"


class ERROR_SCHEMA_ID_IS_NOT_DEFINED(ERROR_INVALID_ARGUMENT):
    _message = (
        "Schema ID is not defined. (schema_id={schema_id}, schema_type={schema_type})"
    )



import logging
import time

from spaceone.core.auth.jwt.jwt_util import JWTUtil
from spaceone.core import utils

from spaceone.identity.error import ERROR_GENERATE_KEY_FAILURE

_LOGGER = logging.getLogger(__name__)


class KeyGenerator:
    def __init__(
        self,
        prv_jwk: dict,
        domain_id: str,
        owner_type: str,
        audience: str,
        api_key_id: str = None,
        refresh_prv_jwk: dict = None,
    ):
        self.prv_jwk = prv_jwk
        self.domain_id = domain_id
        self.owner_type = owner_type
        self.audience = audience
        self.token_id = api_key_id or utils.random_string(16)
        self.refresh_prv_jwk = refresh_prv_jwk

        self._check_parameter()

    def _check_parameter(self):
        if not (self.prv_jwk and self.domain_id and self.audience):
            raise ERROR_GENERATE_KEY_FAILURE()

    def generate_token(
        self,
        token_type: str,
        expired_at: str = None,
        timeout: int = None,
        role_type: str = None,
        workspace_id: str = None,
        permissions: list = None,
        projects: list = None,
    ) -> str:
        payload = {
            "iss": "spaceone.identity",
            "typ": token_type,
            "own": self.owner_type,
            "did": self.domain_id,
            "aud": self.audience,
            "jti": self.token_id,
            "iat": int(time.time()),
            "ver": "2.0",
        }

        if expired_at:
            payload["exp"] = int(utils.iso8601_to_datetime(expired_at).timestamp())
        elif timeout:
            payload["exp"] = int(time.time() + timeout)

        if role_type:
            payload["rol"] = role_type

        if workspace_id:
            payload["wid"] = workspace_id

        if permissions and len(permissions) > 0:
            payload["permissions"] = permissions

        if projects and len(projects) > 0:
            payload["projects"] = projects

        # self._print_key(payload)
        if token_type == "REFRESH_TOKEN":
            return JWTUtil.encode(payload, self.refresh_prv_jwk)
        else:
            return JWTUtil.encode(payload, self.prv_jwk)

    @staticmethod
    def _print_key(payload: dict):
        _LOGGER.debug(
            f"[KeyGenerator._print_key] generated payload. ( "
            f'iss: {payload.get("iss")}, '
            f'rol: {payload.get("rol")}, '
            f'typ: {payload.get("typ")}, '
            f'own: {payload.get("own")}, '
            f'did: {payload.get("did")}, '
            f'wid: {payload.get("wid")}, '
            f'aud: {payload.get("aud")}, '
            f'exp: {payload.get("exp")}, '
            f'iat: {payload.get("iat")}, '
            f'jti: {payload.get("jti")}, '
            f'projects: {payload.get("projects")},'
            f'permissions: {payload.get("permissions")},'
            f'ver: {payload.get("ver")} )'
        )



import logging
from typing import Tuple

from mongoengine import QuerySet
from spaceone.core.manager import BaseManager

from spaceone.identity.model.user_group.database import UserGroup

_LOGGER = logging.getLogger(__name__)


class UserGroupManager(BaseManager):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.user_group_model = UserGroup

    def create_user_group(self, params: dict) -> UserGroup:
        def _rollback(vo: UserGroup):
            _LOGGER.info(
                f"[create_user_group._rollback] Delete user_group: {vo.user_group_id} ({vo.name})"
            )
            vo.delete()

        user_group_vo = self.user_group_model.create(params)

        self.transaction.add_rollback(_rollback, user_group_vo)

        return user_group_vo

    def update_user_group_by_vo(
        self, params: dict, user_group_vo: UserGroup
    ) -> UserGroup:
        def _rollback(old_data):
            _LOGGER.info(
                f'[update_user_group_by_vo._rollback] Revert Data: {old_data["user_group_id"]}'
            )
            user_group_vo.update(old_data)

        self.transaction.add_rollback(_rollback, user_group_vo.to_dict())
        return user_group_vo.update(params)

    @staticmethod
    def delete_user_group_by_vo(user_group_vo: UserGroup) -> None:
        user_group_vo.delete()

    def get_user_group(
        self, user_group_id: str, domain_id: str, workspace_id: str = None
    ):
        conditions = {"user_group_id": user_group_id, "domain_id": domain_id}

        if workspace_id:
            conditions["workspace_id"] = workspace_id

        return self.user_group_model.get(**conditions)

    def filter_user_groups(self, **conditions) -> QuerySet:
        return self.user_group_model.filter(**conditions)

    def list_user_groups(self, query: dict) -> Tuple[QuerySet, int]:
        return self.user_group_model.query(**query)

    def stat_user_group(self, query: dict) -> dict:
        return self.user_group_model.stat(**query)



from typing import Union, Literal
from pydantic import BaseModel

__all__ = [
    "ProjectGroupCreateRequest",
    "ProjectGroupUpdateRequest",
    "ProjectChangeParentGroupRequest",
    "ProjectGroupDeleteRequest",
    "ProjectGroupGetRequest",
    "ProjectGroupSearchQueryRequest",
    "ProjectGroupStatQueryRequest",
]


class ProjectGroupCreateRequest(BaseModel):
    name: str
    tags: Union[dict, None] = {}
    parent_group_id: Union[str, None] = None
    workspace_id: str
    domain_id: str


class ProjectGroupUpdateRequest(BaseModel):
    project_group_id: str
    name: Union[str, None] = None
    tags: Union[dict, None] = None
    workspace_id: str
    domain_id: str


class ProjectChangeParentGroupRequest(BaseModel):
    project_group_id: str
    parent_group_id: Union[str, None] = None
    workspace_id: str
    domain_id: str


class ProjectGroupDeleteRequest(BaseModel):
    project_group_id: str
    workspace_id: str
    domain_id: str


class ProjectGroupGetRequest(BaseModel):
    project_group_id: str
    workspace_id: Union[str, None] = None
    domain_id: str


class ProjectGroupSearchQueryRequest(BaseModel):
    query: Union[dict, None] = None
    project_group_id: Union[str, None] = None
    name: Union[str, None] = None
    parent_group_id: Union[str, None] = None
    workspace_id: Union[str, None] = None
    domain_id: str


class ProjectGroupStatQueryRequest(BaseModel):
    query: dict
    workspace_id: Union[str, None] = None
    domain_id: str
